local Tablet = AceLibrary("Tablet-2.0")
local L = Rock("LibRockLocale-1.0"):GetTranslationNamespace("FuBar_RoutesFu")

RoutesFu = Rock:NewAddon("RoutesFu", "LibRockConfig-1.0", "LibRockDB-1.0", "LibFuBarPlugin-3.0")

RoutesFu:SetDatabase("RoutesFuDB")

RoutesFu:SetFuBarOption("hasIcon", true)
RoutesFu:SetFuBarOption("iconPath", "Interface\\Addons\\FuBar_RoutesFu\\icon.tga" )
RoutesFu:SetFuBarOption("defaultPosition", "RIGHT")
RoutesFu:SetFuBarOption("tooltipType", "Tablet-2.0")
RoutesFu:SetFuBarOption("clickableTooltip", true )
RoutesFu:SetFuBarOption("tooltipHiddenWhenEmpty", true)

local CR = Cartographer:GetModule("Routes")
local BZ = Rock("LibBabble-Zone-3.0"):GetUnstrictLookupTable()
local BZR = Rock("LibBabble-Zone-3.0"):GetReverseLookupTable()
local optionsTable
local tmptable = {}

function RoutesFu:OnInitialize()
	optionsTable = {
		name = "FuBar_RoutesFu",
		desc = self.notes,
		handler = RoutesFu,
		type = "group",
		args = {
		}
	}
	RoutesFu:SetConfigTable(optionsTable)
	RoutesFu.OnMenuRequest = optionsTable

	-- Happens on first-install for LibFuBarPlugin-3.0, the db has a value of nil for both instead of
	-- true/false values and its not possible to register the icon to be shown by default. Dumb? Yes.
	if not self:IsFuBarIconShown() and not self:IsFuBarTextShown() then
		self:ToggleFuBarIconShown() -- show it
		self:ToggleFuBarTextShown() -- show it
		self:ToggleFuBarTextShown() -- hide it (to force the db to save a value of false, rather than nil)
	end
end

function RoutesFu:OnUpdateFuBarTooltip()
	local zone = GetRealZoneText()
	local header = Tablet:AddCategory(
		"columns", 2
	)
	header:AddLine(
		"text", L["Zone:"],
		"text2", zone,
		"text2R", 1,
		"text2G", 1,
		"text2B", 0
	)

	if BZR[zone] then
		zone = BZR[zone]
	end
	local zone_data = CR.db.account.routes[zone]

	if type(zone_data) == "table" then
		local toggle = Tablet:AddCategory(
			"columns", 2,
			"child_textR", 1,
			"child_textG", 1,
			"child_textB", 0,
			"child_text2R", 1,
			"child_text2G", 1,
			"child_text2B", 1,
			"hideBlankLine", true
		)

		for i = #tmptable, 1, -1 do
			tmptable[i] = nil
		end
		for route_name in pairs(zone_data) do
			tinsert(tmptable, route_name)
		end
		table.sort(tmptable)

		for i = 1, #tmptable do
			local route_name = tmptable[i]
			local route_data = zone_data[route_name]
			local checked = not route_data.hidden

			if route_data.route and #route_data.route > 0 then
				toggle:AddLine(
					"text", " "..route_name,
					"text2", L["%s nodes"]:format(#route_data.route),
					"hasCheck", true,
					"checked", checked,
					"func", "OnClickItem",
					"arg1", self,
					"arg2", zone,
					"arg3", route_name,
					"arg4", checked
				)
			end
		end
	end

	if Cartographer:HasModule("Waypoints") and Cartographer:IsModuleActive("Waypoints") then
		local header = Tablet:AddCategory(
			"columns", 1,
			"child_textR", 1,
			"child_textG", 1,
			"child_textB", 1,
			"child_indentation", 20
		)
		header:AddLine(
			"text", "Cartographer_Waypoints:",
			"noInherit", true
		)
		header:AddLine(
			"text", L["Start using Waypoints"],
			"func", CR.QueueFirstNode,
			"arg1", CR
		)	
		header:AddLine(
			"text", L["Change Waypoint direction"],
			"func", CR.ChangeWaypointDirection,
			"arg1", CR
		)	
		header:AddLine(
			"text", L["Stop using Waypoints"],
			"func", CR.RemoveQueuedNode,
			"arg1", CR
		)	
	end

	Tablet:SetHint(L["Click route to toggle"])
	Tablet:SetTitle(L["Cartographer Routes"])
end

function RoutesFu:OnClickItem(zone, route_name, hidden)
	CR.db.account.routes[zone][route_name].hidden = hidden
	CR.DrawWorldmapLines()
	CR.DrawMinimapLines(true)
end
